<?php
/*
 * Plugin Name:Easy Image Alternate Text
 * Plugin URL:  https://www.alumnionlineservices.com/free-plugins/easy-image-alternate-text/
 * Description: The Easy Image Alternate Text Wordpress plugin automatically generates alternate text for images as they are uploaded to your website and on demand from the various Media Library screens as images are edited and inserted into content.
 * Version:     2.0.2
*Author: AlumniOnline Web Services LLC
*Author URI: https://www.alumnionlineservices.com
 * Text Domain: easy-image-alternate-text
 */
// Exit if called directly.
if ( ! defined( 'ABSPATH' ) ) die;

register_activation_hook( __FILE__, 'easy_image_alternate_text_activate_plugin' );
register_uninstall_hook( __FILE__, 'easy_image_alternate_text_uninstall_plugin');

// activate plugin
function easy_image_alternate_text_activate_plugin(){

// deactivate pro version    
if ( is_plugin_active('easy-image-alternate-text-pro/easy-image-alternate-text-pro.php') ) {     
     wp_clear_scheduled_hook('puc_cron_check_updates-easy-image-alternate-text-pro'); 
    deactivate_plugins('easy-image-alternate-text-pro/easy-image-alternate-text-pro.php');    
    }  
}

// uninstall plugin
function easy_image_alternate_text_uninstall_plugin(){
global $wpdb;
    
 if ( is_multisite()) {	
// Get all blogs in the network and activate plugin on each one
$blog_ids = $wpdb->get_col( "SELECT blog_id FROM $wpdb->blogs" );
foreach ( $blog_ids as $blog_id ) {
    switch_to_blog( $blog_id );
 foreach ( wp_load_alloptions() as $option => $value ) {
    if ( strpos( $option, 'easy_image_alternate_text_' ) === 0) {
delete_option( $option ); 
    } 
 }
 restore_current_blog();
}
} else {
foreach ( $blog_ids as $blog_id ) {

 foreach ( wp_load_alloptions() as $option => $value ) {
if ( strpos( $option, 'easy_image_alternate_text_' ) === 0) {
delete_option( $option ); 
    } 
 }
 
 }
}     	   
}
/*********************************************************
 * Add a setting slink to Plugins screen.
**********************************************************/
function easy_image_alternate_text_plugin_action_links( $links ) {
	$link = sprintf(
		'<a href="%s">%s</a>',
		esc_url( admin_url( 'options-media.php' ) ),
		esc_html__( 'Settings', 'easy-image-alternate-text' )
	);

	array_unshift( $links, $link );

	return $links;
}
add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'easy_image_alternate_text_plugin_action_links' );

/*********************************************************
 * Notify user of API key requirement.
 *********************************************************/
function easy_image_alternate_text_admin_notices() {
	if ( get_option( 'easy_image_alternate_text_api_key' ) || ! current_user_can( 'manage_options' ) || get_option( 'easy_image_alternate_text_api_notice_dismissed' ) ) {
		return false;
	}
	?>

	<div class="easy-image-alternate-text-api-notice notice notice-warning is-dismissible">
		<p>
			<?php
				
					_e( 'Additional setup is required to enable automatic alternate text lookup. Go to ','easy-image-alternate-text'); 
				echo '<a href="'.esc_url(get_admin_url()).'options-media.php">';
			_e( 'Media Settings','easy-image-alternate-text'); 			
				echo '</a>';
				_e( ' and enter a valid Microsoft Azure Computer Vision API Key and endpoint.', 'easy-image-alternate-text');
				
			?>

			<?php
			printf(
				esc_html__( 'If you don\'t have them yet, %1$s.', 'easy-image-alternate-text' ),
				sprintf(
					'<a href="https://www.microsoft.com/cognitive-services/en-us/computer-vision-api" target="_blank">%1$s</a>',
					esc_html__( 'register for a Microsoft Azure Computer Vision API Key', 'easy-image-alternate-text' )
				)
			);
			?>
		</p>

		<script type="text/javascript" >
			jQuery( '.easy-image-alternate-text-api-notice' ).on( 'click', '.notice-dismiss', function() {
				jQuery.post( ajaxurl, { "action": "easy_image_alternate_text_dismiss_api_notice" } );
			} );
		</script>
	</div>

	<?php
}
add_action( 'admin_notices', 'easy_image_alternate_text_admin_notices' );

/***********************************************************
 * Remember API key notice dismissal.
 **********************************************************/
function easy_image_alternate_text_dismiss_api_notice() {
	update_option( 'easy_image_alternate_text_api_notice_dismissed', 1 );
	wp_die();
}
add_action( 'wp_ajax_easy_image_alternate_text_dismiss_api_notice', 'easy_image_alternate_text_dismiss_api_notice' );

/*********************************************************
Register settings and fields.
**********************************************************/
function easy_image_alternate_text_register_settings() { 
    
	add_settings_section( 'easy-image-alternate-text-settings', __( 'Easy Image Alternate Text', 'easy-image-alternate-text' ), 'easy_image_alternate_text_settings_section', 'media' );
    
	register_setting( 'media', 'easy_image_alternate_text_endpoint', array('type' => 'string','sanitize_callback' =>'easy_image_alternate_text_validate_url')  );
	add_settings_field( 'easy-image-alternate-text-endpoint', __( 'Endpoint URL', 'easy-image-alternate-text' ), 'easy_image_alternate_text_endpoint_field', 'media', 'easy-image-alternate-text-settings', array( 'label_for' => 'easy_image_alternate_text_endpoint' ) );
    
	register_setting( 'media', 'easy_image_alternate_text_api_key', array('type' => 'string','sanitize_callback' =>'easy_image_alternate_text_validate_apikey') );
	add_settings_field( 'easy-image-alternate-text-api-key', __('API Key:', 'easy-image-alternate-text'), 'easy_image_alternate_text_api_key_field', 'media', 'easy-image-alternate-text-settings', array( 'label_for' => 'easy_image_alternate_text_api_key' ) );
    
	register_setting( 'media', 'easy_image_alternate_text_confidence', array('type' => 'string','sanitize_callback' =>'easy_image_alternate_text_validate_confidence') );
	add_settings_field( 'easy-image-alternate-text-confidence', __( 'Confidence threshold', 'easy-image-alternate-text' ), 'easy_image_alternate_text_confidence_field', 'media', 'easy-image-alternate-text-settings', array( 'label_for' => 'easy_image_alternate_text_confidence' ) );
}

add_action( 'admin_init', 'easy_image_alternate_text_register_settings' );

/**********************************************
// validate url
**********************************************/
function easy_image_alternate_text_validate_url($value){
  if(filter_var($value, FILTER_VALIDATE_URL)) return esc_url_raw($value); 
else return '';  
}

/**********************************************
// validate apikey
**********************************************/
function easy_image_alternate_text_validate_apikey($value){
$value = trim($value);
if(preg_match("/^([[:alnum:]]+)+$/i",$value)) return  sanitize_text_field($value); 
else return '';
}

/**********************************************
// validate confidence level
**********************************************/
function easy_image_alternate_text_validate_confidence($value){
$value = trim($value);
if(is_numeric($value)) return (int)$value; 
else return '50';
}

/**********************************************************
Settings section.
**********************************************************/
function easy_image_alternate_text_settings_section() {
printf(
		__( 'The following options allow alternate text to be automatically added to images. To enable this feature %1$s and enter it below.', 'easy-image-alternate-text' ),
		sprintf(
			'<a href="https://azure.microsoft.com/en-au/services/cognitive-services/computer-vision/">%1$s</a>',
			__( 'Register for a Microsoft Azure Computer Vision API Key ', 'easy-image-alternate-text' )
		)
	); 
}

/**********************************************************
 Field for API Endpoint URL.
 *********************************************************/
function easy_image_alternate_text_endpoint_field() {
$setting = get_option( 'easy_image_alternate_text_endpoint', '' );

echo '<input type="text" size="45" name="easy_image_alternate_text_endpoint" id="easy_image_alternate_text_endpoint" value="'.esc_url($setting).'" />';

}

/*********************************************************
 * Field for API key setting.
 **********************************************************/
function easy_image_alternate_text_api_key_field() {
	$setting = get_option( 'easy_image_alternate_text_api_key' );    
    
 $setting = get_option('easy_image_alternate_text_api_key','');
echo '<input type="text" size="25" name="easy_image_alternate_text_api_key" id="easy_image_alternate_text_api_key" value="'.esc_attr($setting).'" />';
}

/****************************************************************
Field for Confidence threshold setting.
 *****************************************************************/
function easy_image_alternate_text_confidence_field() {
$setting = get_option( 'easy_image_alternate_text_confidence','50' );
echo '<p>';
_e('To control the accuracy of the alt text that is returned, set a confidence level between 0-100. Higher numbers will result in more accurate results but may limit the number of image alt text that can be automatically determined.', 'easy-image-alternate-text');	
echo '<br />';
echo '<input type="text" size="25" name="easy_image_alternate_text_confidence" id="easy_image_alternate_text_confidence" value="'.esc_attr($setting).'" min="0" max="100" />';
echo '</p>';
}

/******************************************************************
Sanitize confidence threshold value.
 *****************************************************************/
function easy_image_alternate_text_sanitize_confidence( $input ) {
	return absint( max( min( $input, 100 ), 0 ) );
}

/*****************************************************************************
Set the alt text.
 *****************************************************************************/
add_action( 'add_attachment', 'easy_image_alternate_text_add_alt_text', 20, 2 );
function easy_image_alternate_text_add_alt_text( $attachment_id ) {
    
if (!get_option( 'easy_image_alternate_text_api_key' )) return;    
	
	/* Only attempt to get a caption for supported image formats. */
	if ( ! in_array( get_post_mime_type( $attachment_id ), array( 'image/jpg', 'image/jpeg', 'image/png', 'image/gif', 'image/bmp', 'image/x-windows-bmp' ) ) ) {
		return false;
	}

	/* Only attempt to get a caption for images under 4MB (Microsoft's limit) */
	if ( filesize( get_attached_file( $attachment_id ) ) > 4000000 ) {
		return false;
	}

	/* Request caption and update meta if one is received. */
	if ( $alt_text = easy_image_alternate_text_get_caption( $attachment_id) ) {
		if(!strstr($alt_text,__('ResponseError','easy-image-alternate-text'))){
		update_post_meta( $attachment_id, '_wp_attachment_image_alt', $alt_text );
		return $alt_text;
		}else{
            return false;
		}
	} else {
		return false;
	}
}

/*****************************************************************************
Get an image caption from Microsoft's Cognitive Services Computer Vision API
for a given attachment.
 *****************************************************************************/
function easy_image_alternate_text_get_caption( $attachment_id) {
if (!get_option( 'easy_image_alternate_text_api_key' )) return;
    
	/* Get the URL for the attachment image. */
    if(is_numeric($attachment_id))
	$image_url = wp_get_attachment_image_url( $attachment_id, 'large' );
    else $image_url = $attachment_id;

	/* Bail if we don't have a URL. */
	if ( ! wp_http_validate_url($image_url) ) {
		return __('ResponseError: image src url could not be found.','easy-image-alternate-text');
	}

	/* Get endpoint from settings, use trial URL as default fallback for those without a saved endpoint. */
	$endpoint = get_option( 'easy_image_alternate_text_endpoint', '' );
    
    $api_key = get_option( 'easy_image_alternate_text_api_key' );

    
	if ( ! $endpoint) {
		return __('ResponseError: endpoint is required','easy-image-alternate-text');
	}

	/* Support newer endpoint URLs that are missing the API endpoint. */
	$has_endpoint = strpos( $endpoint, 'vision/v1.0' );

	if ( $has_endpoint === false ) {
		$endpoint = trailingslashit( $endpoint ) . 'vision/v1.0';
	}

	/* Escape and add describe endpoint. */
	$url = esc_url_raw( trailingslashit( $endpoint ) . 'describe' );

	/* Make API request. */
	$response = wp_remote_post( $url, array(
		'body'    => '{"url" : "' . esc_url($image_url) . '"}',
		'headers' => array(
			'Content-Type'              => 'application/json',
			'Ocp-Apim-Subscription-Key' => esc_attr($api_key),
		),
	) );

	/* Bail on non-200 response. */
	if ( 400 == wp_remote_retrieve_response_code( $response ) ) {
		//update_option( 'easy_image_alternate_text_api_key', '' );
		return __('ResponseError: Invalid image format.','easy-image-alternate-text');
	}
  elseif ( 200 != wp_remote_retrieve_response_code( $response ) ) {
		//update_option( 'easy_image_alternate_text_api_key', '' );
		return __('ResponseError: Failed API lookup. Verify api key and endpoint and subscription status.','easy-image-alternate-text');
	}

	/* Get first caption from response. */
	$response_body = json_decode( wp_remote_retrieve_body( $response ), true );
	if(!isset($response_body['description']['captions'][0])) return false;
    $caption = $response_body['description']['captions'][0] ?: false;

	/* Get confidence threshold as a float. */
	$confidence = get_option( 'easy_image_alternate_text_confidence', 50 ) / 100;

	/* Return the caption if valid and meets confidence threshold. */
	if ( isset( $caption['text'] ) && isset( $caption['confidence'] ) && $caption['confidence'] > $confidence ) {
		return sanitize_text_field( ucfirst( $caption['text'] ) );
	} else {
		return false;
	}
}

/************************************************
add links to media library list to add alt text
***********************************************/
add_filter('media_row_actions', 'easy_image_alternate_text_add_link', 10, 2);
function easy_image_alternate_text_add_link($actions, $post){
    
if (!get_option( 'easy_image_alternate_text_api_key' )) return $actions;    
    
if ( !current_user_can( "edit_pages" ) or !strstr($post->post_mime_type,'image/')) return $actions;
	
$title = __('Look up and apply ALT Text using Microsoft Azure.','easy-image-alternate-text');	
$actions['lookupalt'] = '<a href="'.esc_url(get_site_url()).'/wp-admin/post.php?post='.esc_attr($post->ID).'&action=edit&easy_image_alternate_text_applyalt='.esc_attr($post->ID).'" title="'.esc_attr($title).'" target="_blank" aria-label="Apply ALT Text opens in a new window">'.__('Apply ALT Text', 'easy-image-alternate-text').' <i class="fa fa-external-link" aria-hidden="true"></i></a>';
    
return $actions;
}
/************************************************
// process alt text lookup
************************************************/
if(strstr($_SERVER['REQUEST_URI'],'/wp-admin/post.php') and isset($_GET['easy_image_alternate_text_applyalt']) and get_option( 'easy_image_alternate_text_api_key' ) != '') {
$easy_image_alternate_text_applyalt = (int)$_GET['easy_image_alternate_text_applyalt'];   
$easy_image_alternate_text_returned_caption = easy_image_alternate_text_get_caption($easy_image_alternate_text_applyalt);
// Set the image Alt-Text
if($easy_image_alternate_text_returned_caption != "" and !strstr($easy_image_alternate_text_returned_caption,__('ResponseError','easy-image-alternate-text'))) {
update_post_meta( $easy_image_alternate_text_applyalt, '_wp_attachment_image_alt',
    $easy_image_alternate_text_returned_caption );
      add_action( 'admin_notices', 'easy_image_alternate_text_success_notice' );
}
	elseif(strstr($easy_image_alternate_text_returned_caption,__('ResponseError','easy-image-alternate-text'))) add_action( 'admin_notices', 'easy_image_alternate_text_failure_api_notice' );
	else add_action( 'admin_notices', 'easy_image_alternate_text_failure_notice' );
}
/************************************************
rest add alt text
************************************************/
function easy_image_alternate_text_rest_add_alt_text() {
	if(!get_option( 'easy_image_alternate_text_api_key') or !isset($_GET['easy_image_alternate_text_applyalt'])) return;
    
	check_ajax_referer('wp_rest', '_wpnonce');
	
	
	$attachment_id = (int)$_GET['easy_image_alternate_text_applyalt']; 
			
	return easy_image_alternate_text_get_caption( $attachment_id );
	
}

/***************************************************
display status notice
****************************************************/
function easy_image_alternate_text_success_notice() {
   echo '<p class="notice notice-success is-dismissible">'.__('ALT Text successfully identified. Please verify that it is accurate before continuing.','easy-image-alternate-text');
   
}
function easy_image_alternate_text_failure_notice() {
    echo '<p class="notice notice-error is-dismissible">'.__('ALT Text could not be identified. Try lowering the confidence level under plugin settings.','easy-image-alternate-text').'</p>'; 
}
function easy_image_alternate_text_failure_api_notice() {
    echo '<p class="notice notice-error is-dismissible">'.__('ALT Text could not be identified.','easy-image-alternate-text').'</p>'; 
}

/****************************************************************************************
// add link to Image details screen, displayed when editing image on classic editor screen
***************************************************************************************/
add_action( 'print_media_templates', function() { 
	//return ;
if (!get_option( 'easy_image_alternate_text_api_key' )) return;  
?> 

   <script>
    jQuery(document).ready( function( $ ) {
            
        wp.media.view.ImageDetails = wp.media.view.ImageDetails.extend({
			 
			resetFocus: function() {
                 var attachment = this.options.attachment;
                var domain = '<?php echo esc_url(get_site_url()); ?>';
                if(typeof(attachment.get) === 'function' && this.$('.image').children('img').attr('src').search(domain)>0){
				var button = '<p><button class="easy-image-alternate-text-btn" data-id="'+attachment.get( 'id' )+'"><?php _e("Apply ALT Text", "easy-image-alternate-text-btn"); ?></button></p>';
                }
                else{
                 var button = '<p><button class="easy-image-alternate-text-btn" data-id="'+ this.$('.image').children('img').attr('src')+'"><?php _e("Apply ALT Text", "easy-image-alternate-text"); ?></button></p>';   
                }
               
                   this.$('.image').append(button);
     
            
            }  
        });
      
    });
   </script> 
<?php  }, 10, 1);


/**************************************************************************************
// add image alt lookup button
****************************************************************************************/
add_filter('attachment_fields_to_edit', 'easy_image_alternate_text_add_alt_text_field', 10, 2);
function easy_image_alternate_text_add_alt_text_field($form_fields, $attachment){
if (!get_option( 'easy_image_alternate_text_api_key' )) return $form_fields;      
    // check for an audio attachment
    if ( substr($attachment->post_mime_type, 0, 5) == 'image' ) {
          $form_fields["alttextlookup"] = array(
            "label" => "Apply ALT",
            "input" => "html",
            "html" => '<button type="button"  class="easy-image-alternate-text-btn" data-id="'.$attachment->ID.'">'.__("Apply ALT Text", "easy-image-alternate-text").'</button>',
        );
}

return $form_fields;
}

/*********************************************************
// register endpoint to query alt text via jquery
**********************************************************/
add_action( 'rest_api_init', function () {
  register_rest_route( 'easy_image_alternate_text/v1', '/MScognitiveAPI/', array(
    'methods' => 'GET',
    'callback' => 'easy_image_alternate_text_rest_add_alt_text',
	  'permission_callback' => function () {
			return current_user_can( 'edit_pages' );
		}
  ));
} );

/************************************************
Register and enqueue scripts
************************************************/
add_action( 'admin_enqueue_scripts', 'easy_image_alternate_text_scripts',99); 
function easy_image_alternate_text_scripts(){
if (get_option( 'easy_image_alternate_text_api_key' ) != ""){ 
	
// style sheet	
wp_register_style( 'easy-image-alternate-text-style',  plugin_dir_url( __FILE__ ) .  'style.css', array(), filemtime( plugin_dir_path( __FILE__ ) .  'style.css' ) );	
wp_enqueue_style( 'easy-image-alternate-text-style' );		
	
// scripts to use jquery to set alt text		
wp_register_script( 'easy-image-alternate-text-script',  plugin_dir_url( __FILE__ ) .  'scripts.js', array('jquery'), filemtime( plugin_dir_path( __FILE__ ) .  'scripts.js' ) );
wp_enqueue_script( 'easy-image-alternate-text-script' );
wp_localize_script( 'easy-image-alternate-text-script', 'EIATVariables',array(
	'success' => __("ALT Text successfully identified. Please verify that it is accurate before continuing.", "easy-image-alternate-text"),
    'failure' =>__("ALT Text could not be identified. Try lowering the confidence level under plugin settings.", "easy-image-alternate-text"),
    'failureinternal' =>__("WP rest call failed. Check your browser console for more details.", "easy-image-alternate-text"),
    'resturl' => esc_url_raw(get_rest_url()),
	'nonce' => wp_create_nonce( 'wp_rest' )
	));	
}
}
?>